﻿Imports System.IO
Imports System.Text
Imports System.Data.SqlClient
Imports DRUpdater.drAPI

' ***********************************************************************************************************************************
' Das Interface geht von einer Datenbanktabelle mit der folgenden Struktur aus:

' CREATE TABLE [dbo].[ksAmazonListing](
'   [ProductID] [varchar](20) Not NULL,
'   [ProductIDType] [smallint] Not NULL,
'   [ProductReference] [varchar](20) Not NULL,
'   [DistinctiveTitle] [varchar](250) Not NULL,
'   [ProductAvailability] [smallint] Not NULL,
'   [PriceAmount] [Decimal](9, 2) Not NULL,
'   [isUVP] [bit] Not NULL,
'   [TaxPercent] [Decimal](3, 1) Not NULL,
'   [StockOnHand] [Int] Not NULL,
'   [ProductSize] [varchar](50) NULL,
'   [ProductWeight] [Int] NULL,
'   [Infos] [varchar](50) NULL,
'   [Rabatt] [Decimal](6, 2) Not NULL,
'   [ItemGroup] [varchar](20) Not NULL,
'   [isAmazon] [bit] Not NULL,
'   [FirstUpload] [DateTime] Not NULL,
'   [LastUpdate] [DateTime] Not NULL,
'   [ListingAction] [smallint] Not NULL,
'   Constraint [PK_ksAmazonListing] PRIMARY KEY CLUSTERED ([ProductID] Asc)
'              With (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
') ON [PRIMARY]

' Gelesen werden alle Zeilen mit ListingAction = 2, diese wurden zuvor individuell markiert als relevant für Update via API
' DRUpdater.exe bearbeitet alle Zeilen der Trefferliste, die in drAPI.new() gelesen und in einer DataTable gespeichert wird.
' ***********************************************************************************************************************************

Module DRUpdater

   Public Enum ImportExitCode As Integer
      Success = 0
      UnknownResult = -1
      ParameterError = -2
      InitializationError = -3
      InfileError = -4
      DatabaseError = -5
      RuntimeError = -6
   End Enum

   ' current version
   Dim currentVersionID As String
   Dim currentVersionDate As String

   ' Verbindung für Projekte, aus Angaben in Parametern
   Dim dataConn As SqlConnection

   ' Fehlermeldung aus Unterroutinen für Ausgabe ins Protokoll
   Dim errortext As String = String.Empty

   ' logging definition
   Dim SystemLogFile As String = String.Empty
   Dim ImportLogDirectory As String = String.Empty
   Dim logfile As String = String.Empty
   Dim currday As String = Year(Now) & Right("0" & Month(Now).ToString, 2) & Right("0" & Day(Now).ToString, 2)
   Dim currtime As String = Right("0" & Hour(Now).ToString, 2) & Right("0" & Minute(Now).ToString, 2) & Right("0" & Second(Now).ToString, 2)

   ' separation line to be printed in log
   Dim startProduct As Boolean
   Dim sepline As String = vbCrLf & "--------------------------------------------------------------------------------" & vbCrLf

   ' Connection credentials
   Dim DBServer As String = String.Empty
   Dim DBDatabase As String = String.Empty
   Dim DBUser As String = String.Empty
   Dim DBPassword As String = String.Empty

   ' data containers
   Public drData As drAPI
   Public import As drAPI.importControl


   Sub Main()

      Dim message As String = String.Empty

      ' set culture to ensure proper processing of dates and currency
      My.Application.ChangeCulture("de-DE")
      My.Application.ChangeUICulture("de-DE")

      ' ****************************************************************************************************
      ' *** preparation and checking of technical preliminaries, params, etc.
      ' ****************************************************************************************************

      currentVersionID = My.Application.Info.Version.ToString
      currentVersionDate = File.GetLastWriteTime(My.Application.Info.DirectoryPath & "/" & My.Application.Info.AssemblyName & ".exe").ToShortDateString

      SystemLogFile = Path.Combine(My.Application.Info.DirectoryPath, currday & "_DRUpdater_SysLog.txt")

      message = vbCrLf &
                "*************************************************************************************" & vbCrLf &
                "*** Start DRUpdater V" & currentVersionID & " vom " & currentVersionDate & vbCrLf &
                "*** " & Left(Now.ToString, 19) & vbCrLf &
                "*************************************************************************************" & vbCrLf & vbCrLf
      WriteSysLog(message)

      ' check commandline parameters
      If My.Application.CommandLineArgs.Count < 5 Then
         message = "*** Ungültige Aufrufparameter: " & Join(My.Application.CommandLineArgs.ToArray, "; ") & vbCrLf &
                   "*** Erforderlich: [DBServer] [Datenbank] [DBUser] [DBPwd] [LogDirectory] Optional:['test']" & vbCrLf & "Programm wird beendet"
         WriteSysLog(message)
         Environment.Exit(ImportExitCode.ParameterError)
      End If

      ' save commandline parameters
      DBServer = My.Application.CommandLineArgs(0).Trim
      DBDatabase = My.Application.CommandLineArgs(1).Trim
      DBUser = My.Application.CommandLineArgs(2).Trim
      DBPassword = My.Application.CommandLineArgs(3).Trim

      Dim scsb As New SqlConnectionStringBuilder
      scsb.DataSource = DBServer
      scsb.InitialCatalog = DBDatabase
      scsb.UserID = DBUser
      scsb.Password = DBPassword
      import.ConnectionString = scsb.ConnectionString

      ' check data connection
      If Not checkConnection(dataConn, import.ConnectionString) Then
         WriteSysLog("*** Die Datenbankverbindung für die Parameter kann nicht initialisiert werden: " & errortext)
         Environment.Exit(ImportExitCode.DatabaseError)
      End If
      dataConn.Close()

      ' more params, optionally to be moved to database
      import.ProtocolInputRecord = True
      import.ExtentedProtocol = True
      import.StartAtRecord = 0
      import.MaxRecordsToProcess = 0


      ' ****************************************************************************************************
      ' *** prepare log file and parameter
      ' ****************************************************************************************************

      ImportLogDirectory = My.Application.CommandLineArgs(4).Trim
      If ImportLogDirectory.StartsWith("""") Then ImportLogDirectory = Right(ImportLogDirectory, ImportLogDirectory.Length - 1)
      If ImportLogDirectory.EndsWith("""") Then ImportLogDirectory = Left(ImportLogDirectory, ImportLogDirectory.Length - 1)

      ' start import logfile
      If Not Directory.Exists(ImportLogDirectory) Then
         message = "*** Verzeichnis für Logdatei '" & ImportLogDirectory & "' nicht gefunden oder nicht zugreifbar, Programm wird beendet"
         WriteSysLog(message)
         exitProgram(ImportExitCode.InitializationError)
      End If

      logfile = Path.Combine(ImportLogDirectory, currday & "_" & currtime & "_DRUpdater.txt")
      WriteLog(message)

      ' delete obsolete logs
      cleanupLogs()


      ' ****************************************************************************************************
      ' *** prepare data processing
      ' ****************************************************************************************************

      drData = New drAPI(import)

      If Not drData.isInitialized Then
         import.ErrorMessage = "*** Fehler beim Initialisieren des Dream Robot Interface: " & drData.errorMessage
         WriteSysLog(import.ErrorMessage)
         exitProgram(ImportExitCode.InitializationError)
      End If

      ' log parameter
      message = "Protokolldatei:                " & logfile & vbCrLf &
                "Datenbank:                     " & DBServer & "." & DBDatabase & vbCrLf &
                "Anzahl Produkte zu bearbeiten: " & import.NbrOfProductsInFile & vbCrLf

      ' optionally set test mode
      ' testmode will avoid updating items on DR and sends no püma requests
      ' however, authentification is performed and an access token is stored
      If My.Application.CommandLineArgs.Count > 5 AndAlso My.Application.CommandLineArgs(5).Trim().ToLower = "test" Then

         drData.testMode = True

         message &= vbCrLf &
             "*************************************************************************************" & vbCrLf &
             "*** TEST TEST TEST TEST TEST - Kein Update via DreamRobot Rest API " & vbCrLf &
             "*************************************************************************************" & vbCrLf
      End If


      If import.NbrOfProductsInFile > 0 Then
         message &= vbCrLf & vbCrLf &
             "*************************************************************************************" & vbCrLf &
             "*** Meldungen zu den einzelnen Produkten: " & vbCrLf &
             "*************************************************************************************" & vbCrLf
      End If

      WriteLog(message)


      ' ****************************************************************************************************
      ' *** process single records
      ' ****************************************************************************************************

      ' loop through record lines
      While (drData.nextProductToProcess <= import.NbrOfProductsInFile) And (import.NbrOfProductsProcessed < drData.maxProductToProcess)

         ' indicate separator line in protocol
         startProduct = True

         ' count records processed
         import.NbrOfProductsProcessed += 1

         ' output inrecord to protocol if flag is set
         If import.ProtocolInputRecord Then
            drData.RecordIdentifier = String.Empty
            message = "Aktuelles Produkt: " & vbCrLf
            For Each c As DataColumn In drData.getCurrentProduct.Table.Columns
               message &= c.ColumnName & ": " & stringDBNull(drData.getCurrentProduct(c.ColumnName), "") & vbCrLf
            Next
            WriteLog(message)
         End If


         ' move one record forward and prepare product data
         If Not drData.prepareProduct() Then
            WriteSysLog("Fehler bei der Vorbereitung der Produktdaten: " & drData.errorMessage)
            exitProgram(ImportExitCode.RuntimeError)
         End If
         If Not drData.isOK Then
            WriteLog(drData.errorMessage)
            import.NbrOfProductsError += 1
         Else
            If import.ExtentedProtocol Then
               WriteLog("JSON Data prepared: " & vbCrLf & drData.jsonOutput)
            End If
         End If


         ' process product data and update database if needed
         If drData.isOK Then
            If Not drData.processProduct() Then
               WriteSysLog("Fehler bei der Aktualisierung der Produktdaten: " & drData.errorMessage)
               exitProgram(ImportExitCode.RuntimeError)
            End If
            If Not drData.isOK Then
               WriteLog(drData.errorMessage)
               import.NbrOfProductsError += 1
            Else
               If import.ExtentedProtocol Then
                  WriteLog("DreamRobot Update erfolgreich")
               End If
            End If
         End If


         ' trigger cia
         If drData.isOK Then
            If Not drData.triggerCIA() Then
               WriteSysLog("Fehler beim Triggern des Bestandsabgleichs: " & drData.errorMessage)
               exitProgram(ImportExitCode.RuntimeError)
            End If
            If Not drData.isOK Then
               WriteLog(drData.errorMessage)
               import.NbrOfProductsError += 1
            Else
               If import.ExtentedProtocol Then
                  WriteLog("Triggern des Bestandsabgleichs erfolgreich")
               End If
            End If
         End If


         ' print messages if available
         If drData.hasWarnings Then
            WriteLog(drData.warningMessage)
            import.NbrOfProductsWarnings += 1
         End If
         If drData.hasInformation Then
            WriteLog(drData.infoMessage)
            import.NbrOfProductsInformation += 1
         End If

      End While                     ' end process single product/record


      ' ****************************************************************************************************
      ' *** finalize
      ' ****************************************************************************************************

      ' empty messages for final update of import record
      drData.RecordIdentifier = String.Empty
      drData.errorMessage = String.Empty
      drData.warningMessage = String.Empty
      drData.infoMessage = String.Empty


      ' reset listing flag in database
      If import.NbrOfProductsInFile > 0 Then

         If Not checkConnection(dataConn, import.ConnectionString) Then
            WriteSysLog("*** Die Datenbankverbindung kann nicht initialisiert werden: " & errortext)
            Environment.Exit(ImportExitCode.DatabaseError)
         End If

         Try

            Using cmd As New SqlCommand("update ksAmazonListing set ListingAction = 0 where ListingAction = 2", dataConn)
               cmd.ExecuteNonQuery()
            End Using

            If import.ExtentedProtocol Then
               WriteLog("*** ListingAction Flag erfolgreich zurückgesetzt")
            End If

         Catch ex As Exception
            WriteSysLog("*** Fehler beim Rücksetzen des ListingAction Flags: " & errorInfo(ex))
            Environment.Exit(ImportExitCode.DatabaseError)
         End Try

         dataConn.Close()

      End If

      ' update import record and exit
      exitProgram(ImportExitCode.Success)

   End Sub

   ' write import record and exit with appropriate code
   Private Sub exitProgram(ByVal exitCode As ImportExitCode)

      Dim message As String = String.Empty
      drData.RecordIdentifier = String.Empty

      ' close protocol and exit
      message = vbCrLf & sepline &
                "*** Statistik:" & vbCrLf &
                "*** Anzahl Produkte zur Aktualisierung:          " & import.NbrOfProductsInFile.ToString & vbCrLf &
                "*** Anzahl Produkte bearbeitet:                  " & import.NbrOfProductsProcessed.ToString & vbCrLf &
                "*** davon mit Fehlern:                           " & import.NbrOfProductsError.ToString & vbCrLf &
                "*** davon mit Warnungen:                         " & import.NbrOfProductsWarnings.ToString & vbCrLf &
                "*** davon mit Hinweisen:                         " & import.NbrOfProductsInformation.ToString & vbCrLf
      WriteSysLog(message)


      ' normal end without errors
      If exitCode = ImportExitCode.Success Then
         message = vbCrLf &
                          "*******************************************************************************" & vbCrLf &
                          "*** DRUpdater erfolgreich beendet " & Left(Now.ToString, 19) & vbCrLf &
                          "*******************************************************************************" & vbCrLf
         WriteSysLog(message)

         ' return 0 to calling programm
         Environment.Exit(ImportExitCode.Success)

      Else
         ' abnormal end with exitcode indication type of error
         message = vbCrLf &
                          "*******************************************************************************" & vbCrLf &
                          "*** DRUpdater beendet mit ExitCode (" & exitCode & ") " & Left(Now.ToString, 19) & vbCrLf &
                          "*******************************************************************************" & vbCrLf
         WriteSysLog(message)

         Environment.Exit(exitCode)
      End If

   End Sub


   '*********************************************************************************************************
   ' help functions
   '*********************************************************************************************************

   ' check Connection
   Private Function checkConnection(ByRef conn As SqlConnection, ByVal connstr As String) As Boolean

      Try
         If Not IsNothing(conn) Then
            If conn.State = ConnectionState.Open Then Return True
            If conn.State = ConnectionState.Broken Or conn.State = ConnectionState.Closed Then
               conn.Close()
            End If

            conn.Dispose()
            conn = Nothing
         End If

         conn = New SqlConnection(connstr)
         conn.Open()

      Catch ex As Exception
         errortext = "in checkConnection: " & errorInfo(ex)
         Return False

      End Try

      Return True

   End Function

   ' writes message to system log and import log
   Private Sub WriteSysLog(ByVal LogString As String)

      ' nothing to do if logfile is not set or logstring is empty
      If String.IsNullOrEmpty(SystemLogFile) Then Exit Sub
      If String.IsNullOrEmpty(LogString) Then Exit Sub

      Try

         Dim message As String = LogString

         ' preparing logstring
         If (Not IsNothing(drData)) AndAlso (Not String.IsNullOrEmpty(drData.RecordIdentifier)) Then
            message = drData.RecordIdentifier & ": " & message
         End If
         If Not message.EndsWith(vbCrLf) Then message &= vbCrLf
         If startProduct Then message = vbCrLf & message

         ' logging to system log
         File.AppendAllText(SystemLogFile, message, Encoding.GetEncoding("Windows-1252"))

         ' logging to import log
         WriteLog(LogString)


      Catch ex As Exception
         ' errors in writing to log are logged to windows application eventlog

         ' create event source if not exists
         If Not Diagnostics.EventLog.SourceExists("OHB.ONIXEngine.DRUpdater") Then
            Diagnostics.EventLog.CreateEventSource("OHB.ONIXEngine.DRUpdater", "Application")
         End If

         Dim errLog As New EventLog()
         errLog.Source = "OHB.ONIXEngine.DRUpdater"
         errLog.WriteEntry("Fehler in WriteSysLog beim Schreiben in " & SystemLogFile & ":" & vbCrLf & errorInfo(ex) & vbCrLf &
                           "Auslösende Meldung: " & vbCrLf & LogString, EventLogEntryType.Error)
         errLog.Close()

      End Try

   End Sub

   ' writes message to import log
   Public Sub WriteLog(ByVal LogString As String)

      ' nothing to do if logfile is not set or logstring is empty
      If String.IsNullOrEmpty(logfile) Then Exit Sub
      If String.IsNullOrEmpty(LogString) Then Exit Sub

      Try

         ' preparing logstring
         If (Not IsNothing(drData)) AndAlso (Not String.IsNullOrEmpty(drData.RecordIdentifier)) Then
            LogString = drData.RecordIdentifier & ": " & LogString
         End If
         If Not LogString.EndsWith(vbCrLf) Then LogString &= vbCrLf

         ' logging to import log
         If startProduct Then
            File.AppendAllText(logfile, sepline, Encoding.GetEncoding("Windows-1252"))
            startProduct = False
         End If
         File.AppendAllText(logfile, LogString, Encoding.GetEncoding("Windows-1252"))

      Catch ex As Exception
         ' errors in writing to log are logged to windows application eventlog

         ' create event source if not exists
         If Not Diagnostics.EventLog.SourceExists("OHB.ONIXEngine.DRUpdater") Then
            Diagnostics.EventLog.CreateEventSource("OHB.ONIXEngine.DRUpdater", "Application")
         End If

         Dim errLog As New EventLog()
         errLog.Source = "OHB.ONIXEngine.DRUpdater"
         errLog.WriteEntry("Fehler in WriteLog beim Schreiben in " & logfile & ":" & vbCrLf & errorInfo(ex) & vbCrLf &
                           "Auslösende Meldung: " & vbCrLf & LogString, EventLogEntryType.Error)
         errLog.Close()

      End Try

   End Sub

   ' delete system and import logs older 1 month
   Private Sub cleanupLogs()

      Dim deleteDate As Date = DateAdd(DateInterval.Month, -1, Today())
      Dim lastLogKept As Long = Convert.ToInt32(Year(deleteDate).ToString & Right("0" & Month(deleteDate).ToString, 2) & Right("0" & Day(deleteDate).ToString, 2))
      Dim filename As String
      Dim pathname As String

      ' syslog
      pathname = Directory.GetParent(SystemLogFile).FullName
      For Each f As String In Directory.GetFiles(pathname, "????????_DRUpdater_SysLog.txt")
         filename = Right(f, f.Length - f.LastIndexOf("\") - 1)
         If IsNumeric(Left(filename, 8)) Then
            If CLng(Left(filename, 8)) < lastLogKept Then
               File.Delete(f)
            End If
         End If
      Next

      ' import log
      pathname = Directory.GetParent(logfile).FullName
      For Each f As String In Directory.GetFiles(pathname, "????????_??????_DRUpdater.txt")
         filename = Right(f, f.Length - f.LastIndexOf("\") - 1)
         If IsNumeric(Left(filename, 8)) Then
            If CLng(Left(filename, 8)) < lastLogKept Then
               File.Delete(f)
            End If
         End If
      Next

   End Sub

End Module
